package com.mgh.pmdm.parques.view.ui

import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Toast
import androidx.core.widget.doAfterTextChanged
import androidx.fragment.app.activityViewModels
import androidx.lifecycle.ViewModelProvider
import com.google.android.material.snackbar.Snackbar
import com.mgh.pmdm.parques.model.Park
import com.mgh.pmdm.parques.R
import com.mgh.pmdm.parques.databinding.FragmentSecondBinding
import com.mgh.pmdm.parques.view.dialogs.MiDialogFragment
import com.mgh.pmdm.parques.viewmodel.MainViewModel


class SecondFragment : Fragment(), MiDialogFragment.OnOKOrCancelListener {

    ////val args: SecondFragmentArgs by navArgs()

    private var _binding: FragmentSecondBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    // Referencia al ViewModel
    //private lateinit var secondFragmentViewModel: SecondFragmentViewModel
    private lateinit var viewModel: MainViewModel




    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        _binding = FragmentSecondBinding.inflate(inflater, container, false)
        populateSpinner()
        return binding.root

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Instanciamos el ViewModel, utilizando la clase ViewModelProvider
        // En lugar de this, vamos a utilizar requireActivity(), para trabajar en el ámbito
        // de la actividad y poder compartir el ViewModel
        viewModel = ViewModelProvider(requireActivity()).get(MainViewModel::class.java)


        Log.d("OnViewCreated <<<<<<", viewModel.currentPark.value.toString())
        // Obtenemos el Park mediante arguments

        // Comprobamos si nos pasan datos, y los proporcionamos al ViewModel
        /*val park: Park? = arguments?.getSerializable("park") as Park?

        park?.also {
            viewModel.currentPark.value = it
        }*/


        // Creamos un nuevo método para preparar todos los
        // observers y cómo se actualizan.
        prepareObservers()

        // Gestor del evento onClick sobre el botón de guardar
        binding.btSave.setOnClickListener {

            // Preparamos el diálogo de confirmación
            val miDialogo = MiDialogFragment(
                resources.getString(R.string.SaveData),
                resources.getString(R.string.AskSaveData),
                this
            )

            miDialogo.show(requireActivity().supportFragmentManager, "miDialogo")

        }



    }


    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }


    /*********************************
     * Métodos específicos de la clase
     **********************************/


    fun prepareObservers() {
        // Añadimos observadores al LiveData para ser notificados de los cambios
        viewModel.currentPark.observe(viewLifecycleOwner) {


             if (it!=null)  Log.d("DebugCurrentParkChanged", it.toString())
             else Log.d("DebugCurrentParkChanged","*****")
            //it?.name?.let { it1 -> Log.d("DebugCurrentParkChanged", it1) }


            // Y cuando estos se produzcan, actualizamos las vistas de la interfaz
            it?.let { park ->

                //  TextViews

                binding.Name.setText(park.name)
                binding.Name.doAfterTextChanged { park.name = it.toString() }
                binding.Description.setText(park.desc)
                binding.Description.doAfterTextChanged { park.desc = it.toString() }
                binding.Phone.setText(park.phone)
                binding.Phone.doAfterTextChanged { park.phone = it.toString() }
                binding.webSite.setText(park.webiste)
                binding.webSite.doAfterTextChanged { park.webiste = it.toString() }

                // Actualización del valor de los spinners
                for (i in 0..binding.SpinnerOpeningTime.adapter.count)
                    if (binding.SpinnerOpeningTime.adapter.getItem(i).equals(park.openingTime)) {
                        binding.SpinnerOpeningTime.setSelection(i)
                        break
                    }

                // Listeners para los cambios en los spinners
                binding.SpinnerOpeningTime.onItemSelectedListener = object :
                    AdapterView.OnItemSelectedListener {
                    override fun onItemSelected(
                        parent: AdapterView<*>?,
                        view: View?,
                        position: Int,
                        id: Long
                    ) {
                        it.openingTime = parent?.getItemAtPosition(position).toString()
                    }

                    override fun onNothingSelected(parent: AdapterView<*>?) {
                        // Nada por hacer
                    }
                }



                for (i in 0..binding.SpinnerClosingTime.adapter.count)
                    if (binding.SpinnerClosingTime.adapter.getItem(i).equals(it.closingTime)) {
                        binding.SpinnerClosingTime.setSelection(i)
                        break
                    }

                binding.SpinnerClosingTime.onItemSelectedListener = object :
                    AdapterView.OnItemSelectedListener {
                    override fun onItemSelected(
                        parent: AdapterView<*>?,
                        view: View?,
                        position: Int,
                        id: Long
                    ) {
                        it.closingTime = parent?.getItemAtPosition(position).toString()
                    }

                    override fun onNothingSelected(parent: AdapterView<*>?) {
                        // Nada por hacer
                    }
                }

                // Actualización de los Checkboxes
                if (park.sports == true) binding.cbSport.isChecked = true

                binding.cbSport.setOnCheckedChangeListener { _, isChecked ->
                    park.sports = isChecked
                }

                if (park.children == true) binding.cbChildren.isChecked = true

                binding.cbChildren.setOnCheckedChangeListener { _, isChecked ->
                    park.children = isChecked
                }

                if (park.Bar == true) binding.cbBar.isChecked = true
                binding.cbBar.setOnCheckedChangeListener { _, isChecked ->
                    park.Bar = isChecked
                }

                if (park.Pets == true) binding.cbMascotas.isChecked = true
                binding.cbMascotas.setOnCheckedChangeListener { _, isChecked ->
                    park.Pets = isChecked
                }

            }

            // ImageView, de momento fija
            binding.Image.setImageResource(R.drawable.appimg)

        }
    }


    // Método específico para poblar el spinner para la hora de cierre
    private fun populateSpinner() {
        ArrayAdapter.createFromResource(
            requireActivity(),
            R.array.horas,
            android.R.layout.simple_spinner_dropdown_item
        ).also { adapter ->
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_item)
            binding.SpinnerClosingTime.adapter = adapter
        }
    }

    override fun onPositiveClick() {

        // Creamos un nuevo parque con la información del formulario
        val newPark = Park(
            binding.Name.text.toString(),
            binding.Description.text.toString(),
            binding.Phone.text.toString(),
            binding.webSite.text.toString(),
            binding.SpinnerOpeningTime.selectedItem.toString(),
            binding.SpinnerClosingTime.selectedItem.toString(),
            binding.cbSport.isChecked,
            binding.cbChildren.isChecked,
            binding.cbBar.isChecked,
            binding.cbMascotas.isChecked
        )

        val saved=viewModel.savePark(newPark)

        if (saved){
            Snackbar.make(binding.root, resources.getString(R.string.dataSaved), Snackbar.LENGTH_LONG).show()
        }

    }

    override fun onCancelClick() {
        // Si pulsa cancelar mostramos un toast
        Toast.makeText(
            requireActivity().applicationContext,
            resources.getString(R.string.actionCancelled), Toast.LENGTH_SHORT
        ).show()
    }


}